@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use English qw< -no_match_vars >;
 use Data::Dumper;
-our $VERSION = '0.729';
+our $VERSION = '0.729001';
 
 use Log::Log4perl::Tiny qw< :easy :dead_if_first >;
 
@@ -45,7 +45,10 @@ sub parse_by_format {
 
    my $value = $args{value} //= ['whatever'];
    $value = [$value] unless ref $value;
-   my $multiple = (scalar(@$value) > 1) || ($value->[0] ne 'whatever');
+   my $multiple =
+        (ref($value) ne 'ARRAY')
+     || (scalar(@$value) > 1)
+     || ($value->[0] ne 'whatever');
 
    return parse_by_separators(
       %args,
@@ -105,8 +108,7 @@ sub _resolve_value {
    $value //= 'whatever';
    $value = $value->($args) if ref($value) eq 'CODE';
    my $ref = ref $value;
-   return $value if $ref eq 'Regexp';
-   ($value, $ref) = ([$value], 'ARRAY') unless $ref;
+   ($value, $ref) = ([$value], 'ARRAY') if (!$ref) || ($ref eq 'Regexp');
    LOGCROAK "$args->{name}: unknown value type $ref" if $ref ne 'ARRAY';
 
    my (%flag_for, @regexps);
@@ -285,6 +287,7 @@ sub parse_by_separators {
          die {
             message => "'$name': invalid record, expected $n_keys, "
               . "got $n_values only",
+            values => \@values,
             record => $record
            }
            if $n_values < $n_keys;
@@ -371,21 +374,39 @@ sub parse_by_value_separator {
    my $allow_missing = $args{allow_missing} || 0;
    my $allow_surplus = $args{allow_surplus} || 0;
    my $trim          = $args{trim};
+   my $go_global     = $^V lt v5.18.0;
 
    return sub {
       my $record = shift;
 
       my @values;
-      $record->{$input} =~ m/
-         \A (?: $value $separator (?{push @values, $^N}) )*
-            $value \z (?{push @values, $^N})
-         /gmxs
-        or die {
-         message   => 'invalid record',
-         separator => $separator,
-         value     => $value,
-         record    => $record,
-        };
+      if ($go_global) {
+         local our @global_values = ();
+         my $collector = qr/(?{push @global_values, $^N})/;
+         $record->{$input} =~ m/
+            \A (?: $value $separator $collector )*
+               $value \z $collector
+            /gmxs
+           or die {
+            message   => 'invalid record',
+            separator => $separator,
+            value     => $value,
+            record    => $record,
+           };
+         @values = @global_values;
+      }
+      else {
+         $record->{$input} =~ m/
+            \A (?: $value $separator (?{push @values, $^N}) )*
+               $value \z (?{push @values, $^N})
+            /gmxs
+           or die {
+            message   => 'invalid record',
+            separator => $separator,
+            value     => $value,
+            record    => $record,
+           };
+      }
       trim(@values) if $trim;
       if ($decode) {
          eval { @values = @{$decode->(\@values)}; 1 } or do {
